﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.StorageClient;
using System.Windows.Forms;
using System.IO;
using System.Net;
using System.Xml;
using System.Xml.Xsl;
using System.Security;
using System.Security.Cryptography;
using System.Security.Cryptography.X509Certificates;
using System.Xml.Serialization;

namespace CopyAzureStorage
{
    // This contains all the helpers used by events on the form.
    public class RestAPI
    {
        private String globalAzureSourceAccountID = String.Empty;
        private String certificatePath = String.Empty;

        // Define the global variables for REST API
        private String postBodyString = string.Empty;
        private String apiVersion = "2012-03-01";

        // Logic for REST API Service Call
        internal RestAPICallResponse ProcessRestAPI(RestAPICallRequest singleRestCallDetails)
        {
            // If an Operation Call get the response status
            RestAPICallResponse apiResponse = new RestAPICallResponse();
            apiResponse.HasError = false;
            apiResponse.HasAsync = false;
            apiResponse.ClientID = singleRestCallDetails.ClientID;
            apiResponse.RowPosition = singleRestCallDetails.RowPosition;
            apiResponse.SubscriptionID = singleRestCallDetails.SubscriptionID;
            apiResponse.VirtualMachineID = singleRestCallDetails.VirtualMachineID;

            try
            {
                // Get the URL.  Should be something like https://management.core.windows.net/...
                Uri requestUri = new Uri(singleRestCallDetails.APIURL);

                // Create the request and specify attributes of the request.
                HttpWebRequest request = (HttpWebRequest)HttpWebRequest.Create(requestUri);

                // Define the required headers to specify the API version and operation type.
                request.Headers.Add("x-ms-version", apiVersion);
                request.Method = singleRestCallDetails.APIOperation;
                request.ContentType = "application/xml";

                // If POST set the Body
                if (singleRestCallDetails.APIOperation == "POST")
                {
                    XmlDocument xDoc = new XmlDocument();
                    xDoc.LoadXml(singleRestCallDetails.APIPostText);

                    Stream requestStream = request.GetRequestStream();
                    StreamWriter streamWriter = new StreamWriter(requestStream, System.Text.UTF8Encoding.UTF8);
                    xDoc.Save(streamWriter);

                    streamWriter.Close();
                    requestStream.Close();
                }

                // Get the Certificate
                X509Certificate2 myCert = new X509Certificate2(singleRestCallDetails.APICertificatePath);
                
                // Add the Certificate
                // Could replace with reading from the Certificate Store but I have issues with that
                request.ClientCertificates.Add(myCert);

                // Make the call using the web request.
                HttpWebResponse response = (HttpWebResponse)request.GetResponse();

                HttpStatusCode responseStatus = response.StatusCode;

                switch (singleRestCallDetails.APIMethod.ToUpper())
                {
                    case "GETVMDETAILSIMPORT":
                        {
                            // If it is not found we know the VM does not exist
                            if (responseStatus == HttpStatusCode.NotFound)
                            {
                                apiResponse.VMRoleCount = 0;
                            }
                            else
                            {
                                // If anything but a 404 check to see how many roles we have
                                if (response.ContentLength > 0)
                                {
                                    // Parse the web response.
                                    Stream responseStream = response.GetResponseStream();
                                    StreamReader reader = new StreamReader(responseStream);

                                    XmlDocument xDocResp = new XmlDocument();
                                    xDocResp.Load(reader);

                                    // Select all the roles - assumes these are only VM roles
                                    XmlNodeList docList = xDocResp.SelectNodes("/*[local-name()='Deployment' and namespace-uri()='http://schemas.microsoft.com/windowsazure']/*[local-name()='RoleList' and namespace-uri()='http://schemas.microsoft.com/windowsazure']/*[local-name()='Role' and namespace-uri()='http://schemas.microsoft.com/windowsazure']");

                                    // Set the number of Roles returned - ASSUME only has roles that are VM Roles
                                    apiResponse.VMRoleCount = docList.Count;
                                }
                            }
                        }
                        break;
                    case "GETVMDETAILSEXPORT":
                        if (response.ContentLength > 0)
                        {
                            // Parse the web response.
                            Stream responseStream = response.GetResponseStream();
                            StreamReader reader = new StreamReader(responseStream);

                            XmlDocument xDocResp = new XmlDocument();
                            xDocResp.Load(reader);

                            // Save the RAW output from the API
                            xDocResp.Save(singleRestCallDetails.VMStoragePath + singleRestCallDetails.ServiceName + "_" + singleRestCallDetails.VMName + "_" + singleRestCallDetails.Location + "_BACKUP.xml");

                            // Show the output
                            string vmDeployment = xDocResp.GetElementsByTagName("Name")[0].InnerText;
                            string vmExportSize = xDocResp.GetElementsByTagName("RoleSize")[0].InnerText;

                            string outputFileNameVM = singleRestCallDetails.VMStoragePath + singleRestCallDetails.ServiceName + "_" + vmDeployment + "_" + singleRestCallDetails.VMName + "_" + singleRestCallDetails.Location + "_" + vmExportSize + "_VM.xml";
                            string outputFileNameRole = singleRestCallDetails.VMStoragePath + singleRestCallDetails.ServiceName + "_" + vmDeployment + "_" + singleRestCallDetails.VMName + "_" + singleRestCallDetails.Location + "_" + vmExportSize + "_ROLE.xml";
                            string outputFileNameTemp = singleRestCallDetails.VMStoragePath + singleRestCallDetails.ServiceName + "_" + vmDeployment + "_" + singleRestCallDetails.VMName + "_" + singleRestCallDetails.Location + "_" + vmExportSize + "_TEMP.xml";

                            // Remove the Roles that are not part of this VM
                            // Remove XML Nodes that cause an issue when creating a new VM
                            // To be replaced with XSLT at some point
                            XmlNodeList docList = xDocResp.SelectNodes("/*[local-name()='Deployment' and namespace-uri()='http://schemas.microsoft.com/windowsazure']/*[local-name()='RoleList' and namespace-uri()='http://schemas.microsoft.com/windowsazure']/*[local-name()='Role' and namespace-uri()='http://schemas.microsoft.com/windowsazure']");

                            // Set the number of Roles returned - ASSUME only has roles that are VM Roles
                            apiResponse.VMRoleCount = docList.Count;
                            if (apiResponse.VMRoleCount == 0)
                                throw new Exception("BE101 - No Roles found in Service Export.");

                            // Remove the nodes not for this VM
                            foreach (XmlNode element in docList)
                            {
                                if (!element.InnerXml.ToUpper().Contains(singleRestCallDetails.VMName.ToUpper() + "</ROLENAME>"))
                                {
                                    element.ParentNode.RemoveChild(element);
                                }
                            }

                            // XSLT Transformation
                            // Create a writer for writing the transformed file.
                            XmlWriter writertemp = XmlWriter.Create(outputFileNameTemp);
                            XsltSettings settings = new XsltSettings();
                            settings.EnableScript = true;

                            // Create and load the transform with script execution enabled.
                            XslCompiledTransform transformtemp = new XslCompiledTransform();
                            transformtemp.Load(@"XSLT\GetVM_to_CreateVM.xslt", settings, null);

                            // Execute the transformation.
                            transformtemp.Transform(xDocResp, writertemp);

                            writertemp.Flush();
                            writertemp.Close();
                            writertemp = null;

                            XmlWriter writer = XmlWriter.Create(outputFileNameVM);
                            XmlDocument readertemp = new XmlDocument();
                            readertemp.Load(outputFileNameTemp);

                            XslCompiledTransform transform = new XslCompiledTransform();
                            transform.Load(@"XSLT\RemoveEmptyNodes.xslt", settings, null);

                            // Execute the transformation.
                            transform.Transform(readertemp, writer);

                            writer.Flush();
                            writer.Close();
                            readertemp = null;

                            // Delete the Temp file used in the XSLT
                            File.Delete(outputFileNameTemp);

                            //Create the File needed to add this as a Role
                            XmlWriter writerRole = XmlWriter.Create(outputFileNameRole);
                            XmlDocument readerRole = new XmlDocument();
                            readerRole.Load(outputFileNameVM);

                            XslCompiledTransform transformRole = new XslCompiledTransform();
                            transformRole.Load(@"XSLT\GetVM_to_AddVMRole.xslt", settings, null);

                            // Execute the transformation.
                            transformRole.Transform(readerRole, writerRole);

                            writerRole.Flush();
                            writerRole.Close();

                            // Global Cleanup
                            responseStream.Close();
                            reader.Close();
                        }
                        break;
                    default:
                        // If the response is NULL nothing to show
                        if (response.ContentLength > 0)
                        {
                            // Parse the web response.
                            Stream responseStream = response.GetResponseStream();
                            StreamReader reader = new StreamReader(responseStream);

                            XmlDocument xDocResp = new XmlDocument();
                            xDocResp.Load(reader);

                            // Check the response to see if it is an Operation Message - if so get the Status
                            if (singleRestCallDetails.APIURL.Contains("/operations/"))
                            {
                                apiResponse.Status = xDocResp.SelectSingleNode("/*[local-name()='Operation' and namespace-uri()='http://schemas.microsoft.com/windowsazure']/*[local-name()='Status' and namespace-uri()='http://schemas.microsoft.com/windowsazure']").InnerXml;
                            }

                            // Cleanup
                            responseStream.Close();
                            reader.Close();
                        }
                        break;
                }

                // Get the Request ID and set the URL to check the status if this is not an Operations Check and a 202 response for Async Request
                if (!singleRestCallDetails.APIURL.Contains("/operations/") && responseStatus == HttpStatusCode.Accepted)
                {
                    WebHeaderCollection responseHeaders = response.Headers;
                    apiResponse.HasAsync = true;
                    apiResponse.ASyncId = responseHeaders.Get("x-ms-request-id");
                }

                // CLose the HttpWebResponse
                response.Close();
            }
            catch (WebException exWeb)
            {
                string responseText = "Unable to parse the response content.  It is likely zero bytes.";

                if (exWeb.Response.ContentLength > 0)
                {
                    // Parse the web response.
                    Stream responseStream = exWeb.Response.GetResponseStream();
                    StreamReader reader = new StreamReader(responseStream);

                    XmlDocument xDocResp = new XmlDocument();
                    xDocResp.Load(reader);
                    HttpWebResponse responseStatus = (HttpWebResponse)exWeb.Response;

                    responseText = NiceFormatXml(xDocResp);
                    
                    responseStream.Close();
                    reader.Close();
                }
                
                if (singleRestCallDetails.APIMethod == "GETVMDETAILSIMPORT")
                {
                    // A 404 expected is expected if no roles are found
                    apiResponse.VMRoleCount = 0;
                }
                else
                {
                    apiResponse.HasError = true;
                    apiResponse.ErrorURL = singleRestCallDetails.APIURL;
                    apiResponse.ErrorOperation = singleRestCallDetails.APIOperation;

                    apiResponse.ErrorText = "Web Error making REST API call.  The error message is: " + exWeb.Message + ".  The response is:" + responseText;
                }
            }
            catch (Exception ex)
            {
                apiResponse.HasError = true;
                apiResponse.ErrorURL = singleRestCallDetails.APIURL;
                apiResponse.ErrorOperation = singleRestCallDetails.APIOperation;
                apiResponse.ErrorText = "Error making REST API call.  The error message is: " + ex.Message;
            }

            return apiResponse;
        }


        private string NiceFormatXml(XmlDocument xDoc)
        {
            StringBuilder niceString = new StringBuilder();
            StringWriter strWriter = new StringWriter(niceString);
            XmlTextWriter xmlWriter = new XmlTextWriter(strWriter);

            xmlWriter.Formatting = Formatting.Indented;
            xDoc.WriteTo(xmlWriter);

            xmlWriter.Close();
            strWriter.Close();

            return niceString.ToString();
        }
    }
}
